@@ -35,7 +35,7 @@ import unipath
 from unipath import pathof
 import unicodedata
 
-_launcher_version=20160605
+_launcher_version=20170227
 
 _PKG_VER = re.compile(r'''<\s*package[^>]*version\s*=\s*["']([^'"]*)['"][^>]*>''',re.IGNORECASE)
 
@@ -56,6 +56,7 @@ ext_mime_map = {
                 '.svg'  : 'image/svg+xml',
                 '.xhtml': 'application/xhtml+xml',
                 '.html' : 'application/xhtml+xml',
+                '.htm'  : 'application/xhtml+xml',
                 '.otf'  : 'application/vnd.ms-opentype',
                 #'.otf'  : 'application/x-font-opentype',
                 '.ttf'  : 'application/x-font-ttf',
@@ -69,33 +70,36 @@ ext_mime_map = {
                 '.xml'  : 'application/oebs-page-map+xml',
                 '.opf'  : 'application/oebps-package+xml',
                 '.smil' : 'application/smil+xml',
-                '.pls'  : 'application/pls-xml',
+                '.pls'  : 'application/pls+xml',
+                '.xpgt' : 'application/adobe-page-template+xml',
                 '.js'   : 'text/javascript',
                 #'.js'   : 'application/javascript',
                 '.epub' : 'application/epub+zip',
                 }
 
 mime_base_map = {
-                'image/jpeg'                    : 'Images',
-                'image/png'                     : 'Images',
-                'image/gif'                     : 'Images',
-                'image/svg+xml'                 : 'Images',
-                'application/xhtml+xml'         : 'Text',
-                'application/x-font-ttf'        : 'Fonts',
-                'application/x-font-opentype'   : 'Fonts',
-                'application/vnd.ms-opentype'   : 'Fonts',
-                'application/font-woff'         : 'Fonts',
-                'audio/mpeg'                    : 'Audio',
-                'audio/mp3'                     : 'Audio',
-                'audio/mp4'                     : 'Audio',
-                'video/mp4'                     : 'Video',
-                'text/css'                      : 'Styles',
-                'application/x-dtbncx+xml'      : '',
-                'application/oebps-package+xml' : '',
-                'application/oebs-page-map+xml' : 'Misc',
-                'application/smil+xml'          : 'Misc',
-                'text/javascript'               : 'Misc',
-                'application/pls-xml'           : 'Misc',
+                'image/jpeg'                              : 'Images',
+                'image/png'                               : 'Images',
+                'image/gif'                               : 'Images',
+                'image/svg+xml'                           : 'Images',
+                'application/xhtml+xml'                   : 'Text',
+                'application/x-font-ttf'                  : 'Fonts',
+                'application/x-font-opentype'             : 'Fonts',
+                'application/vnd.ms-opentype'             : 'Fonts',
+                'application/font-woff'                   : 'Fonts',
+                'audio/mpeg'                              : 'Audio',
+                'audio/mp3'                               : 'Audio',
+                'audio/mp4'                               : 'Audio',
+                'video/mp4'                               : 'Video',
+                'text/css'                                : 'Styles',
+                'application/x-dtbncx+xml'                : '',
+                'application/oebps-package+xml'           : '',
+                'application/oebs-page-map+xml'           : 'Misc',
+                'application/smil+xml'                    : 'Misc',
+                'application/adobe-page-template+xml'     : 'Misc',
+                'application/vnd.adobe-page-template+xml' : 'Misc',
+                'text/javascript'                         : 'Misc',
+                'application/pls+xml'                     : 'Misc',
                 }
 
 
@@ -105,6 +109,19 @@ PROTECTED_FILES = [
     'OEBPS/content.opf'
 ]
 
+TEXT_MIMETYPES = [
+                'image/svg+xml',
+                'application/xhtml+xml',
+                'text/css',
+                'application/x-dtbncx+xml',
+                'application/oebps-package+xml',
+                'application/oebs-page-map+xml',
+                'application/smil+xml',
+                'application/adobe-page-template+xml',
+                'application/vnd.adobe-page-template+xml',
+                'text/javascript',
+                'application/pls+xml'
+]
 
 class WrapperException(Exception):
     pass
@@ -122,6 +139,13 @@ class Wrapper(object):
         # initialize the sigil cofiguration info passed in outdir with sigil.cfg
         self.appdir = None
         self.usrsupdir = None
+        # Location of directory containing hunspell dictionaries on Linux
+        self.linux_hunspell_dict_dirs = []
+        # Sigil interface language code
+        self.sigil_ui_lang = None
+        # Default Sigil spell check dictionary
+        self.sigil_spellcheck_lang = None
+        # File selected in Sigil's Book Browser
         self.selected = []
         cfg = ''
         with open(os.path.join(self.outdir, 'sigil.cfg'), 'rb') as f:
@@ -131,6 +155,10 @@ class Wrapper(object):
         if len(cfg_lst) >= 2:
             self.appdir = cfg_lst.pop(0)
             self.usrsupdir = cfg_lst.pop(0)
+            if not sys.platform.startswith('darwin') and not sys.platform.startswith('win'):
+                self.linux_hunspell_dict_dirs = cfg_lst.pop(0).split(":")
+            self.sigil_ui_lang = cfg_lst.pop(0)
+            self.sigil_spellcheck_lang = cfg_lst.pop(0)
             self.selected = cfg_lst
         os.environ['SigilGumboLibPath'] = self.get_gumbo_path()
 
@@ -203,6 +231,7 @@ class Wrapper(object):
     # utility routine to get mime from href
     def getmime(self,  href):
         href = unicode_str(href)
+        href = unquoteurl(href)
         filename = os.path.basename(href)
         ext = os.path.splitext(filename)[1]
         ext = ext.lower()
@@ -442,6 +471,7 @@ class Wrapper(object):
             type = unicode_str(type)
             title = unicode_str(title)
             href = unicode_str(href)
+            href = unquoteurl(href)
             if type not in _guide_types:
                 type = "other." + type
             if title is None:
@@ -500,7 +530,7 @@ class Wrapper(object):
         with open(filepath,'rb') as fp:
             data = fp.read()
         mime = self.id_to_mime.get(id,'')
-        if mime.endswith('+xml'):
+        if mime in TEXT_MIMETYPES:
             data = unicode_str(data)
         return data
 
@@ -516,7 +546,7 @@ class Wrapper(object):
         base = os.path.dirname(filepath)
         if not unipath.exists(base):
             os.makedirs(pathof(base))
-        if mime.endswith('+xml') or isinstance(data, text_type):
+        if mime in TEXT_MIMETYPES or isinstance(data, text_type):
             data = utf8_str(data)
         with open(filepath,'wb') as fp:
             fp.write(data)
@@ -551,7 +581,7 @@ class Wrapper(object):
         base = os.path.dirname(filepath)
         if not unipath.exists(base):
             os.makedirs(base)
-        if mime.endswith('+xml') or isinstance(data, text_type):
+        if mime in TEXT_MIMETYPES or isinstance(data, text_type):
             data = utf8_str(data)
         with open(filepath,'wb') as fp:
             fp.write(data)
@@ -633,20 +663,24 @@ class Wrapper(object):
 
     def map_href_to_id(self, href, ow):
         href = unicode_str(href)
+        href = unquoteurl(href)
         return self.href_to_id.get(href,ow)
 
     def map_basename_to_id(self, basename, ow):
         basename = unicode_str(basename)
         ext = os.path.splitext(basename)[1]
         ext = ext.lower()
-        mime = ext_mime_map.get(ext,None)
+        mime = ext_mime_map.get(ext,"")
         if mime.startswith("audio"):
             base = 'Audio'
         elif mime.startswith("video"):
             base = "Video"
         else:
             base = mime_base_map.get(mime,'Misc')
-        href = base + "/" + basename
+        if base == "":
+            href = basename
+        else:
+            href = base + "/" + basename
         return self.href_to_id.get(href,ow)
 
     def map_id_to_href(self, id, ow):
@@ -671,14 +705,17 @@ class Wrapper(object):
 
 
     # routines to work on ebook files that are not part of an opf manifest
+    # their "id" is actually their unique relative path from book root
 
     def readotherfile(self, book_href):
         id = unicode_str(book_href)
+        id = unquoteurl(id)
         if id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use readfile')
-        # handle special case of trying to read the opf
+        # handle special case of trying to read the opf after it has been modified
         if id is not None and id == "OEBPS/content.opf":
-            return self.build_opf()
+            if id in self.modified:
+                return self.build_opf()
         filepath = self.id_to_filepath.get(id, None)
         if filepath is None:
             raise WrapperException('book href does not exist')
@@ -691,16 +728,17 @@ class Wrapper(object):
         basename = os.path.basename(filepath)
         ext = os.path.splitext(basename)[1]
         ext = ext.lower()
-        mime = ext_mime_map.get(ext,'')
+        mime = ext_mime_map.get(ext,"")
         data = b''
         with open(filepath,'rb') as fp:
             data = fp.read()
-        if mime.endswith('+xml'):
+        if mime in TEXT_MIMETYPES:
             data = unicode_str(data)
         return data
 
     def writeotherfile(self, book_href, data):
         id = unicode_str(book_href)
+        id = unquoteurl(id)
         if id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use writefile')
         filepath = self.id_to_filepath.get(id, None)
@@ -720,6 +758,7 @@ class Wrapper(object):
 
     def addotherfile(self, book_href, data) :
         id = unicode_str(book_href)
+        id = unquoteurl(id)
         if id in self.other:
             raise WrapperException('book href must be unquie')
         desired_path = id.replace("/",os.sep)
@@ -739,6 +778,7 @@ class Wrapper(object):
 
     def deleteotherfile(self, book_href):
         id = unicode_str(book_href)
+        id = unquoteurl(id)
         if id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use deletefile')
         filepath = self.id_to_filepath.get(id, None)
@@ -797,42 +837,11 @@ class Wrapper(object):
             apaths.append(unipath.abspath(os.path.join(self.usrsupdir,"hunspell_dictionaries")))
         else:
             # Linux
-            system_hunspell_dicts = ''
-            share_prefix = ''
-            share_override = ''
-
-            # get the env var SIGIL_DICTIONARIES set at launch time.
-            if 'SIGIL_DICTIONARIES' in os.environ.keys():
-                system_hunspell_dicts = os.environ['SIGIL_DICTIONARIES']
-
-            # Runtime env var override of 'share/sigil' directory.
-            if 'SIGIL_EXTRA_ROOT' in os.environ.keys():
-                share_override = os.environ['SIGIL_EXTRA_ROOT']
-
-            # The sigil launch script in <install_prefix>/bin knows where Sigil's build time
-            # share prefix is and sets the env var SIGIL_SHARE_PREFIX to its value.
-            if 'SIGIL_SHARE_PREFIX' in os.environ.keys():
-                share_prefix = os.environ['SIGIL_SHARE_PREFIX']
-
-            # If someone didn't launch Sigil with its launch script, this may save the
-            # day (as long as the user didn't override SHARE_INSTALL_PREFIX at compile time).
-            if not len(share_prefix) and not len(share_override):
-                share_prefix = unipath.abspath(os.path.join(self.appdir,"..",".."))
-
-            # If the SIGIL_DICTIONARIES env var has content, use it for the dictionary location.
-            if len(system_hunspell_dicts):
-                for path in system_hunspell_dicts.split(':'):
-                    apaths.append(unipath.abspath(path.strip()))
-            else:
-                # If the 'share/sigil' location has indeed been overridden at runtime, use that.
-                if len(share_override):
-                    apaths.append(unipath.abspath(os.path.join(share_override, "hunspell_dictionaries")))
-                else:
-                    # Otherwise, use Sigil's bundled hunspell dictionary location.
-                    apaths.append(unipath.abspath(os.path.join(share_prefix, "share", 'sigil', "hunspell_dictionaries")))
+            for path in self.linux_hunspell_dict_dirs:
+                apaths.append(unipath.abspath(path.strip()))
             apaths.append(unipath.abspath(os.path.join(self.usrsupdir,"hunspell_dictionaries")))
         return apaths
-                        
+
     def get_gumbo_path(self):
         if sys.platform.startswith('darwin'):
             lib_dir = unipath.abspath(os.path.join(self.appdir,"..","lib"))
@@ -844,7 +853,7 @@ class Wrapper(object):
             lib_dir = unipath.abspath(self.appdir)
             lib_name = 'libsigilgumbo.so'
         return os.path.join(lib_dir, lib_name)
-            
+
     def get_hunspell_path(self):
         if sys.platform.startswith('darwin'):
             lib_dir = unipath.abspath(os.path.join(self.appdir,"..","lib"))
@@ -856,4 +865,3 @@ class Wrapper(object):
             lib_dir = unipath.abspath(self.appdir)
             lib_name = 'libhunspell.so'
         return os.path.join(lib_dir, lib_name)
-
